/*
 *  Copyright (C) 2006-2007 Daniel Bessler <danielb@kiba-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Daniel Bessler <danielb@kiba-dock.org>
*/

#ifndef __KIBA_DOCK_H__
#define __KIBA_DOCK_H__

#include "../config.h"

#include <gdk/gdkx.h>
#include <X11/Xlib.h>
#include <cairo.h>
#include <akamaru.h>

#ifdef HAVE_GLITZ
#ifdef CAIRO_HAS_GLITZ_SURFACE
  #define USE_GLITZ
#endif
#endif

#ifdef HAVE_SVG
  #include <librsvg/rsvg.h>
  #include <librsvg/rsvg-cairo.h>
#endif
#ifdef USE_GLITZ
  #include <glitz-glx.h>
  #include <cairo-glitz.h>
#endif

#define KIBA_TYPE_DOCK             (kiba_dock_get_type ())
#define KIBA_DOCK(obj)             (G_TYPE_CHECK_INSTANCE_CAST ((obj), KIBA_TYPE_DOCK, KibaDock))
#define KIBA_DOCK_CLASS(obj)       (G_TYPE_CHECK_CLASS_CAST ((obj), KIBA_DOCK,  KibaDockClass))
#define KIBA_IS_DOCK(obj)          (G_TYPE_CHECK_INSTANCE_TYPE ((obj), KIBA_TYPE_DOCK))
#define KIBA_IS_DOCK_CLASS(obj)    (G_TYPE_CHECK_CLASS_TYPE ((obj), KIBA_TYPE_DOCK))
#define KIBA_DOCK_GET_CLASS        (G_TYPE_INSTANCE_GET_CLASS ((obj), KIBA_TYPE_DOCK, KibaDockClass))

/* flags macro's */
#define SET_FLAG(s,x) (s |= (1 << x))
#define DEL_FLAG(s,x) (s = s & ~(1<<x))
#define GET_FLAG(s,x) (s & (1<<x))

#define KIBA_ERROR kiba_error_quark ()
#define gdkdisplay (Display*) gdk_x11_get_default_xdisplay ()

#define dsettings dock->settings
#define default_icon_size 40

#define SECOND_INTERVAL  1000
#define MINUTE_INTERVAL  60000

#define QUIT_STEPSIZE 10

#define DEBUG(msg); \
  if (KIBA_DOCK (dock)->verbose){ \
    g_print ("%s @ line %d:  ", __FILE__, __LINE__);  \
    g_print msg;}

GQuark kiba_error_quark (void);

typedef struct _KibaDock           KibaDock;
typedef struct _KibaDockClass      KibaDockClass;
typedef struct _KibaText           KibaText;
typedef struct _KibaSettings       KibaSettings;
typedef struct _KibaObject         KibaObject;
typedef struct _KibaRemObject      KibaRemObject;
typedef struct _KibaPlugin         KibaPlugin;
typedef struct _PluginFunc         PluginFunc;
typedef struct _PluginHandle       PluginHandle;

/* plugin functions */
/* initiall plugin */
typedef void       (*init_t)            (KibaDock*, KibaPlugin*);
/* free plugin */
typedef void       (*free_t)            (KibaDock*);
/* render plugin */
typedef void       (*paint_t)           (KibaDock*, KibaObject*, cairo_t*, GdkRectangle);
/* update surface buffer for plugin */
typedef void       (*update_surface_t)  (KibaDock*, KibaObject*, cairo_t*, gboolean);
/* plugin option changed */
typedef void       (*option_t)          (KibaDock*, const char*, const char*, const char*);
/* button release event */
typedef void       (*release_t)         (KibaDock*, KibaObject*, int);
/* scroll event */
typedef void       (*scroll_t)          (KibaDock*, KibaObject*, int);
/* right click menu */
typedef void       (*menu_t)            (KibaDock*, KibaObject*, GtkMenu*);


typedef enum {
  KIBA_DOCK_POSITION_LEFT_EDGE_TOP,
  KIBA_DOCK_POSITION_LEFT_EDGE_CENTER,
  KIBA_DOCK_POSITION_LEFT_EDGE_BOTTOM,

  KIBA_DOCK_POSITION_RIGHT_EDGE_TOP,
  KIBA_DOCK_POSITION_RIGHT_EDGE_CENTER,
  KIBA_DOCK_POSITION_RIGHT_EDGE_BOTTOM,

  KIBA_DOCK_POSITION_TOP_EDGE_LEFT,
  KIBA_DOCK_POSITION_TOP_EDGE_CENTER,
  KIBA_DOCK_POSITION_TOP_EDGE_RIGHT,

  KIBA_DOCK_POSITION_BOTTOM_EDGE_LEFT,
  KIBA_DOCK_POSITION_BOTTOM_EDGE_CENTER,
  KIBA_DOCK_POSITION_BOTTOM_EDGE_RIGHT
} KibaDockPosition;

enum {
  KIBA_QUIT,
  KIBA_ENTERED,
  KIBA_PHYSICS,
  KIBA_DRAGGING,
  KIBA_HIDING,
  KIBA_MOVING,
};

typedef struct {
  const char     *name;
  GtkOrientation  orientation;
  int             halignment;
  int             valignment;
  GdkPoint        anchor;
} KibaDockLayoutData;

static const KibaDockLayoutData kiba_layout_data[] = {
  { "left_edge_top",      GTK_ORIENTATION_VERTICAL,   0, 0, { 0, 1 } },
  { "left_edge_center",   GTK_ORIENTATION_VERTICAL,   0, 1, { 0, 2 } },
  { "left_edge_bottom",   GTK_ORIENTATION_VERTICAL,   0, 2, { 0, 3 } },
  { "right_edge_top",     GTK_ORIENTATION_VERTICAL,   2, 0, { 4, 1 } },
  { "right_edge_center",  GTK_ORIENTATION_VERTICAL,   2, 1, { 4, 2 } },
  { "right_edge_bottom",  GTK_ORIENTATION_VERTICAL,   2, 2, { 4, 3 } },
  { "top_edge_left",      GTK_ORIENTATION_HORIZONTAL, 0, 0, { 1, 0 } },
  { "top_edge_center",    GTK_ORIENTATION_HORIZONTAL, 1, 0, { 2, 0 } },
  { "top_edge_right",     GTK_ORIENTATION_HORIZONTAL, 2, 0, { 3, 0 } },
  { "bottom_edge_left",   GTK_ORIENTATION_HORIZONTAL, 0, 2, { 1, 4 } },
  { "bottom_edge_center", GTK_ORIENTATION_HORIZONTAL, 1, 2, { 2, 4 } },
  { "bottom_edge_right",  GTK_ORIENTATION_HORIZONTAL, 2, 2, { 3, 4 } }
};

struct _KibaPlugin {
  char   *name;            /* plugin name (example: "clock") */
  int     position;        /* plugin position in the dock */
  void   *plugin_struct;   /* plugin struct (example: KibaClock*) */
  void   *option_struct;   /* plugin option struct (example: KibaClockOptions*) */
  GList  *plugin_data;     /* used for plugin to plugin communication, content depends on Plugin */
};

struct _PluginFunc
{
  char             *plugin_name;         /* plugin name (example: "clock") */
  init_t            init_func;           /* kiba_plugin_init function */
  option_t          option_changed_func; /* kiba_plugin_option_changed function */
  paint_t           paint_func;          /* kiba_plugin_paint function */
  update_surface_t  update_surface_func; /* kiba_plugin_update_surface function */
  menu_t            menu_func;           /* kiba_plugin_menu function */
  release_t         release_func;        /* kiba_plugin_release function */
  scroll_t          scroll_func;         /* kiba_plugin_scroll function */
  free_t            free_func;           /* kiba_plugin_free function */
};

struct _KibaDock
{
  GtkWidget           widget;           /* main widget */
  KibaText           *text;             /* titel proberty struct */
  KibaSettings       *settings;         /* settings struct */
  int                 flags;            /* state FLAGS */

  /* object related variables */
  GList              *objects;           //List of all kiba-objects, content: KibaObject's
  int                 num_objects;       //number of objects
  GList              *plugins;           //List of all running Plugins, content: KibaPlugins's
  GList              *plugin_functions;  //List contains functions from the plugins, content: PluginFunc's
  GList              *plugin_handles;    //saves the opened librarys

  KibaObject         *dragging_object;   //the currently dragged object or NULL
  KibaObject         *prelight_object;   //the currently prelighted object or NULL
  KibaObject         *exec_object;       //the current exec object or NULL
  KibaObject         *text_object;       //the current text object or NULL

  /* physics engine related variables */
  AkamaruModel        model;
  AkamaruAnchor      *anchor;              //anchor for object->spring
  AkamaruAnchor      *mouse_anchor;        //anchor for object dragging
  AkamaruAnchor      *last_rope_anchor;    //anchor for last object gravity
  AkamaruAnchor      *first_rope_anchor;   //anchor for first object gravity

  /* rendering related variables */
  GdkPixmap          *pixmap;
  GdkGC              *gc;
  cairo_surface_t    *surface;
  cairo_surface_t    *prelight_surface;
  cairo_surface_t    *progress_surface;
  cairo_surface_t    *triangle_surface;
  cairo_surface_t    *arrow_surface;
  gboolean            surface_needs_update;          /* TRUE if the dock Surface needs an update */
  gboolean            prelight_surface_needs_update; /* TRUE if the prelight Surface needs an update */
  gboolean            progress_surface_needs_update; /* TRUE if the prelight Surface needs an update */
  gboolean            triangle_surface_needs_update; /* TRUE if the triangle Surface needs an update */
  gboolean            arrow_surface_needs_update;    /* TRUE if the arrow Surface needs an update */
  gboolean            window_shaped;                 /* TRUE if the Window is Input shaped */
#ifdef USE_GLITZ
  glitz_drawable_format_t  *glitz_draw_format;
  glitz_drawable_t         *glitz_drawable;
  glitz_format_t           *glitz_format;
#endif

  /* options */
  char               *settings_file;      /* stores the setting file content. */

  /* geometry variables */
  KibaDockPosition    position;
  KibaDockLayoutData *d;
  GdkRectangle        geometry;           //monitor/screen geometry
  GdkRectangle        window_geometry;    /* geometry of the window */
  GdkRectangle        last_mouse_position;
  GtkAllocation       allocation;         //dock-size
  GtkAllocation       dest_allocation;    //dock-size
  GtkAllocation       last_allocation;    //dock-size before allocation
  gboolean            rerender;
  int                 drag_distance;       /* distance of object dragging */
  int                 dx, dy;
  int                 hide_requests;

  /* object remove animation */
  GList              *removed_objects;      /* content: KibaRemObject`s */

  /* animated bg */
  double              bg_offset;
  double              old_bg_offset;

  /* drag animation */
  GdkRectangle        drag_geometry;
  gboolean            drag_motion;
  double              drag_window_offset;
  double              drag_alpha;

  /* parameter */
  char               *cwd;               //current working directory
  gboolean            benchmark;
  gboolean            verbose;
  gint                redraws;
  gint                events;
};

struct _KibaDockClass
{
  GtkWidgetClass parent_class;
};

struct _KibaText
{
  GdkRectangle     geometry;          /* saves the aim geometry */
  GdkRectangle     geometry_animate;  /* saves the geometry while animating */
  double           fade_alpha;        /* stores the fade_alpha (0.0 when not shown) */
  char            *text;              /* stores the last rendered text */
};

/* reload the dock */
void
kiba_reload (KibaDock *dock);

void
kiba_get_rgb_from_string (const char *html_color,
                          double     *r,
                          double     *g,
                          double     *b);

void
kiba_get_rgba_from_string (const char *html_color,
                           double     *r,
                           double     *g,
                           double     *b,
                           double     *alpha);

/* create right click menu */
void
kiba_create_menu (KibaDock       *dock,
                  KibaObject     *object,
                  GdkEventButton *event);

/* returns the KibaPlugin for the given plugin_name */
KibaPlugin*
kiba_get_plugin (KibaDock   *dock,
                 const char *plugin_name);

/* returns the plugin struct for the given plugin_name */
void*
kiba_get_plugin_struct (KibaDock   *dock,
                        const char *plugin_name);

/* finds the closest plugin position related to mousecursor position for a given plugin */
void
kiba_plugin_find_closest_position (KibaDock    *dock,
                                   KibaObject  *object);

/* update size for all objects of plugin */
void
kiba_plugin_size_changed (KibaDock   *dock,
                          KibaPlugin *plugin,
                          double      size);

/* try to run the given command */
gint
kiba_exec (gpointer data); //data must by type KibaDock*

/* copy fromfile to tofile, the file must be a textfile */
int
copyfile (char *fromfile,
          char *tofile);

/* returns content of /proc/$PID/cmdline */
char*
kiba_get_command_from_pid (int pid);

#endif
